@*******************************
@* GBA Graphics Library        *
@*  by Jeff Frohwein           *
@* http://www.devrs.com        *
@*******************************
@ v1.0223 - Original release

@ Use fill2.s instead since it is faster than this version

@ Solid fill an area in GBA graphics modes 3/5 in ARM asm
@ Entry: x = start X coordinate
@        y = start Y coordinate
@       fc = fill color
@       bc = boundary color
@     addr = screen base addr
@ (void) gfxFill1 (u8 x, u8 y, u16 fc, u16 bc, u32 addr);

        .ARM
        .ALIGN
        .GLOBL  gfxFill1

gfxFill1:
        sub     sp,sp,#4         @ protect the stack so addr doesn't
                                 @  doesn't get overwritten.
        stmfd   r13!,{r0-r8,r14}

        ldr     r8,[r13,#44]     @ Get the screen base address

        mov     r7,#0            @ set points on stack count to 0

1:      bl      9f               @ Is point filled?
        bcs     4f               @ yes

2:      sub     r1,r1,#1
        bl      9f               @ Is point above this one set?
        bcc     2b               @ No, so check again
        add     r1,r1,#1

3:      sub     r0,r0,#1
        bl      9f               @ Is point on the left set ?
        bcc     2b               @ No, so check above again
        add     r0,r0,#1

@ Plot a pixel at r0,r1 of color r2

        stmfd   r13!,{r3-r5,r14}
        mov     r4,#480
        mul     r5,r4,r1
        add     r5,r5,r0,lsl #1
        add     r4,r5,r8
        strh    r2,[r4]
        ldmfd   r13!,{r3-r5,r14}

        add     r1,r1,#1
        bl      9f               @ Is point below set ?
        bcc     5f               @ No

        sub     r1,r1,#1
        add     r0,r0,#1
        bl      9f               @ Is point on the right set ?
        bcc     2b               @ No
        sub     r0,r0,#1

@ Are there any more points on stack?

4:      cmp     r7,#0
        beq     8f               @ No

@ Get point on stack

        ldmfd   r13!,{r0,r1}

        sub     r7,r7,#1
        b       1b

5:
        sub     r1,r1,#1
        add     r0,r0,#1
        bl      9f               @ Is point on the right set ?
        bcs     6f               @ Yes

        add     r1,r1,#1
        bl      9f               @ Is lower right point set ?
        bcc     7f               @ No

        sub     r1,r1,#1

@ Save point position on stack

        add     r7,r7,#1         @ Increment points on stack count

@@@@ ADD STACK OVERFLOW TEST HERE @@@@@

        stmfd   r13!,{r0,r1}     @ Save point on stack

@ Point below is not set

6:      add     r1,r1,#1
7:      sub     r0,r0,#1
        b       3b

@ Exit fill routine

8:
        ldmfd   r13!,{r0-r8,r14}
        add     sp,sp,#4
        bx      r14

@ Clear Carry if pixel r0,r1 needs to be filled

9:
        cmps    r0,#240         @ is x coord out of bounds?
        bxcs    lr              @ yes
        cmps    r1,#160         @ is y coord out of founds?
        bxcs    lr              @ yes

        mov     r4,#480
        mul     r5,r4,r1
        add     r5,r5,r0,lsl #1
        add     r4,r5,r8
        ldrh    r6,[r4]

        cmps    r6,r2           @ Have we hit a filled spot?
        bxeq    lr              @ yes, exit with carry set

        cmps    r6,r3           @ Have we hit the boundary?
        bxeq    lr              @ yes, exit with carry set

@ Clear carry flag
        movs    r5,r0,lsl #1
        bx      lr






